/*****************************************************************************************
   
   Copyright (c) Microsoft Corporation. All rights reserved.

   Use of this code sample is subject to the terms of the Microsoft
   Public License, a copy of which should always be distributed with
   this file.  You can also access a copy of this license agreement at:
   http://www.microsoft.com/opensource/licenses.mspx#Ms-PL

 ****************************************************************************************/
using System;

namespace Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary
{
    public class Field
    {
        public enum ContentTypes
        {
            text = 0,
            xml = 1,
            html = 2,
            xhtml = 3
        }

        public enum Types
        {
            none = 0,
            number = 1,
            floatingpoint = 2,
            datetime = 3,
            boolean = 4,
        }

        #region private variables

        private System.Xml.XmlElement m_XmlElement;
        private System.Xml.XmlElement m_ExtensionXmlElement;

        private string m_Element;
        private string m_Label;
        private string m_DefaultValue;
        private string m_PickListID;

        private bool m_Required;
        private bool m_Hidden;
        private bool m_Array;
        private bool m_ReadOnly;
        private bool m_Multiline;

        private Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Binding m_Binding;
        private Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes m_ContentType;
        private Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types m_Type;

        #endregion


        public Field(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Binding i_Binding, string i_Element)
            : this(i_Binding, System.String.Empty, i_Element, System.String.Empty, System.String.Empty, false, false, false, false, false, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.text, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.none)
        {
        }

        public Field(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Binding i_Binding, string i_PickListID, string i_Element)
            : this(i_Binding, i_PickListID, i_Element, System.String.Empty, System.String.Empty, false, false, false, false, false, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.text, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.none)
        {
        }

        public Field(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Binding i_Binding, string i_Element, string i_Label, bool i_Required, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes i_ContentType)
            : this(i_Binding, System.String.Empty, i_Element, i_Label, System.String.Empty, i_Required, false, false, false, false, i_ContentType, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.none)
        {
        }

        public Field(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Binding i_Binding, string i_PickListID, string i_Element, string i_Label, bool i_Required, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes i_ContentType)
            : this(i_Binding, i_PickListID, i_Element, i_Label, System.String.Empty, i_Required, false, false, false, false, i_ContentType, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.none)
        {
        }

        public Field(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Binding i_Binding, string i_PickListID, string i_Element, string i_Label, string i_DefaultValue, bool i_Required, bool i_Hidden, bool i_Array, bool i_ReadOnly, bool i_Multiline, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes i_ContentType, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types i_Type)
        {
            this.FieldInitialize
                (
                i_PickListID,
                i_Element,
                i_Label,
                i_DefaultValue,
                i_Required,
                i_Hidden,
                i_Array,
                i_ReadOnly,
                i_Multiline,
                i_ContentType,
                i_Type
                );

            string ElementName = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.FIELD_ELEMENT_NAME;
            if (!System.String.IsNullOrEmpty(i_Binding.NamespacePrefix))
                ElementName = i_Binding.NamespacePrefix + ":" + ElementName;

            m_XmlElement = i_Binding.XmlElement.OwnerDocument.CreateElement
                (
                ElementName,
                Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.SIMPLE_FEED_BINDINGS_XML_NAMESPACE_URI
                );

            m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.ELEMENT_ATTRIBUTE, m_Element);

            if (!System.String.IsNullOrEmpty(m_PickListID))
                m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.PICKLIST_ID_ATTRIBUTE, m_PickListID);

            if (m_ContentType != Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.text)
            {
                string ContentType;

                switch (m_ContentType)
                {
                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.xml:
                        {
                            ContentType = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.XML_CONTENT_TYPE;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.html:
                        {
                            ContentType = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.HTML_CONTENT_TYPE;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.xhtml:
                        {
                            ContentType = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.XHTML_CONTENT_TYPE;
                            break;
                        }

                    default:
                        {
                            throw new System.ArgumentException("Invalid value for m_ContentType!");
                        }
                }

                m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.CONTENT_TYPE_ATTRIBUTE, ContentType);
            }

            if (m_Type != Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.none)
            {
                string Type;

                switch (m_Type)
                {
                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.number:
                        {
                            Type = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.NUMBER_TYPE;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.floatingpoint:
                        {
                            Type = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.FLOATINGPOINT_TYPE;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.datetime:
                        {
                            Type = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.DATETIME_TYPE;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.boolean:
                        {
                            Type = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.BOOLEAN_TYPE;
                            break;
                        }

                    default:
                        {
                            throw new System.ArgumentException("Invalid value for m_Type!");
                        }
                }

                m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.TYPE_ATTRIBUTE, Type);
            }

            if (!System.String.IsNullOrEmpty(m_Label))
                m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.LABEL_ATTRIBUTE, m_Label);

            if (!System.String.IsNullOrEmpty(m_DefaultValue))
                m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.DEFAULT_VALUE_ATTRIBUTE, m_DefaultValue);

            if (m_Required)
                m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.REQUIRED_ATTRIBUTE, true.ToString().ToLower());

            if (m_Hidden)
                m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.HIDDEN_ATTRIBUTE, true.ToString().ToLower());

            if (m_ReadOnly)
                m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.READ_ONLY_ATTRIBUTE, true.ToString().ToLower());

            if (m_Array)
                m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.ARRAY_ATTRIBUTE, true.ToString().ToLower());

            if (m_Multiline)
                m_XmlElement.SetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.MULTILINE_ATTRIBUTE, true.ToString().ToLower());
        }

        public Field(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Binding i_Binding, System.Xml.XmlElement i_FieldXmlElement)
        {
            if (i_FieldXmlElement == null)
                throw new System.ArgumentException("i_FieldXmlElement must be non-null!");

            bool InvalidElement =
                (i_FieldXmlElement.LocalName != Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.FIELD_ELEMENT_NAME) ||
                (i_FieldXmlElement.NamespaceURI != Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.SIMPLE_FEED_BINDINGS_XML_NAMESPACE_URI);

            if (InvalidElement)
                throw new System.ArgumentException("Invalid i_FieldXmlElement!");

            m_XmlElement = i_FieldXmlElement;

            string Element = m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.ELEMENT_ATTRIBUTE);

            string PickListID = System.String.Empty;
            if (m_XmlElement.HasAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.PICKLIST_ID_ATTRIBUTE))
                PickListID = m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.PICKLIST_ID_ATTRIBUTE);

            string ContentType = System.String.Empty;
            if (m_XmlElement.HasAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.CONTENT_TYPE_ATTRIBUTE))
                ContentType = m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.CONTENT_TYPE_ATTRIBUTE);

            string Type = System.String.Empty;
            if (m_XmlElement.HasAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.TYPE_ATTRIBUTE))
                Type = m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.TYPE_ATTRIBUTE);

            string Label = System.String.Empty;
            if (m_XmlElement.HasAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.LABEL_ATTRIBUTE))
                Label = m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.LABEL_ATTRIBUTE);

            string DefaultValue = System.String.Empty;
            if (m_XmlElement.HasAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.DEFAULT_VALUE_ATTRIBUTE))
                DefaultValue = m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.DEFAULT_VALUE_ATTRIBUTE);

            bool Required = false;
            if (m_XmlElement.HasAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.REQUIRED_ATTRIBUTE))
                Required = System.Convert.ToBoolean(m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.REQUIRED_ATTRIBUTE));

            bool Hidden = false;
            if (m_XmlElement.HasAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.HIDDEN_ATTRIBUTE))
                Hidden = System.Convert.ToBoolean(m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.HIDDEN_ATTRIBUTE));

            bool ReadOnly = false;
            if (m_XmlElement.HasAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.READ_ONLY_ATTRIBUTE))
                ReadOnly = System.Convert.ToBoolean(m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.READ_ONLY_ATTRIBUTE));

            bool Array = false;
            if (m_XmlElement.HasAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.ARRAY_ATTRIBUTE))
                Array = System.Convert.ToBoolean(m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.ARRAY_ATTRIBUTE));

            bool Multiline = false;
            if (m_XmlElement.HasAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.MULTILINE_ATTRIBUTE))
                Multiline = System.Convert.ToBoolean(m_XmlElement.GetAttribute(Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.MULTILINE_ATTRIBUTE));

            this.FieldInitialize
                (
                PickListID,
                Element,
                Label,
                DefaultValue,
                Required,
                Hidden,
                Array,
                ReadOnly,
                Multiline,
                ContentType,
                Type
                );

            System.Xml.XmlElement ExtensionXmlElement = (System.Xml.XmlElement)i_FieldXmlElement.SelectSingleNode
                (
                i_Binding.NamespacePrefix + ":" + Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.FIELD_EXTENSION_ELEMENT_NAME,
                i_Binding.XmlNamespaceManager
                );

            this.AddExtensionXmlElement(ExtensionXmlElement, false);
        }

        internal void FieldInitialize(string i_PickListID, string i_Element, string i_Label, string i_DefaultValue, bool i_Required, bool i_Hidden, bool i_Array, bool i_ReadOnly, bool i_Multiline, string i_ContentType, string i_Type)
        {
            Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes ContentType = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.text;
            Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types Type = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.none;

            if (!System.String.IsNullOrEmpty(i_ContentType))
            {
                switch (i_ContentType)
                {
                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.TEXT_CONTENT_TYPE:
                        {
                            ContentType = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.text;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.XML_CONTENT_TYPE:
                        {
                            ContentType = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.xml;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.HTML_CONTENT_TYPE:
                        {
                            ContentType = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.html;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.XHTML_CONTENT_TYPE:
                        {
                            ContentType = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes.xhtml;
                            break;
                        }

                    default:
                        {
                            throw new System.ArgumentException("Invalid value for i_ContentType!");
                        }
                }
            }

            if (!System.String.IsNullOrEmpty(i_Type))
            {
                switch (i_Type)
                {
                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.NONE_TYPE:
                        {
                            Type = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.none;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.NUMBER_TYPE:
                        {
                            Type = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.number;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.FLOATINGPOINT_TYPE:
                        {
                            Type = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.floatingpoint;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.DATETIME_TYPE:
                        {
                            Type = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.datetime;
                            break;
                        }

                    case Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Constants.BOOLEAN_TYPE:
                        {
                            Type = Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types.boolean;
                            break;
                        }

                    default:
                        {
                            throw new System.ArgumentException("Invalid value for i_Type!");
                        }
                }
            }

            this.FieldInitialize
                (
                i_PickListID,
                i_Element,
                i_Label,
                i_DefaultValue,
                i_Required,
                i_Hidden,
                i_Array,
                i_ReadOnly,
                i_Multiline,
                ContentType,
                Type
                );
        }

        internal void FieldInitialize(string i_PickListID, string i_Element, string i_Label, string i_DefaultValue, bool i_Required, bool i_Hidden, bool i_Array, bool i_ReadOnly, bool i_Multiline, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes i_ContentType, Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types i_Type)
        {
            if (System.String.IsNullOrEmpty(i_Element) || (i_Element.IndexOf("\\") != -1))
                throw new System.ArgumentException("Invalid value for i_Element!");

            string[] ElementNames = i_Element.Split('/');
            string[] NamespaceIDs = new string[0];

            m_Element = i_Element;
            m_PickListID = i_PickListID;

            //  Strip off trailing "/" if it is the last character
            if (m_Element.LastIndexOf("/") == m_Element.Length - 1)
                m_Element = m_Element.Substring(0, m_Element.Length - 1);

            if (System.String.IsNullOrEmpty(i_Label))
                m_Label = m_Element;
            else
                m_Label = i_Label;

            m_DefaultValue = i_DefaultValue;

            m_Required = i_Required;
            m_Hidden = i_Hidden;
            m_Array = i_Array;
            m_ReadOnly = i_ReadOnly;
            m_Multiline = i_Multiline;
            m_ContentType = i_ContentType;
            m_Type = i_Type;
        }

        public Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Binding Binding
        {
            get
            {
                return m_Binding;
            }

            internal set
            {
                if (value == null)
                    throw new System.ArgumentException("Value must be non-null!");

                m_Binding = value;
            }
        }

        public string Element
        {
            get
            {
                return m_Element;
            }
        }

        public string PickListID
        {
            get
            {
                return m_PickListID;
            }
        }

        public Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.ContentTypes ContentType
        {
            get
            {
                return m_ContentType;
            }
        }

        public Microsoft.CSA.FeedSync.SimpleFeedBindingsLibrary.Field.Types Type
        {
            get
            {
                return m_Type;
            }
        }

        public string Label
        {
            get
            {
                return m_Label;
            }
        }

        public string DefaultValue
        {
            get
            {
                return m_DefaultValue;
            }
        }

        public bool Array
        {
            get
            {
                return m_Array;
            }
        }

        public bool Required
        {
            get
            {
                return m_Required;
            }
        }

        public bool Hidden
        {
            get
            {
                return m_Hidden;
            }
        }

        public bool ReadOnly
        {
            get
            {
                return m_ReadOnly;
            }
        }

        public bool Multiline
        {
            get
            {
                return m_Multiline;
            }
        }

        public System.Xml.XmlElement XmlElement
        {
            get
            {
                return m_XmlElement;
            }
        }

        public System.Xml.XmlElement ExtensionXmlElement
        {
            get
            {
                return m_ExtensionXmlElement;
            }
        }

        public void AddExtensionXmlElement(System.Xml.XmlElement i_ExtensionXmlElement)
        {
            this.AddExtensionXmlElement
                (
                i_ExtensionXmlElement,
                true
                );
        }

        private void AddExtensionXmlElement(System.Xml.XmlElement i_ExtensionXmlElement, bool i_AppendXmlElement)
        {
            if (m_ExtensionXmlElement != null)
                throw new System.ArgumentException("Extension already exists for field!");

            m_ExtensionXmlElement = i_ExtensionXmlElement;

            if (i_AppendXmlElement)
                m_XmlElement.AppendChild(i_ExtensionXmlElement);
        }

    }
}
